from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..main import CupLegend

from ..types import BadAPIRequest, Hero, StarBox, Gifts, Gift, Clan, Account


class InfoMethods:
    def __init__(self, parent: "CupLegend"):
        self._parent = parent

    def account(self, UUID: int) -> Account | BadAPIRequest:
        """Возвращает информацию об аккаунте.

        :param UUID: Идентификатор игрока
        :type UUID: int"""
        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"account/{UUID}")

        if result["status_code"] == 200:
            return Account.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def clan(self, UCID: int) -> Clan | BadAPIRequest:
        """Возвращает информацию о клане.

        :param UCID: Идентификатор клана
        :type UCID: int
        """
        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"clan/{UCID}")

        if result["status_code"] == 200:
            return Clan.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def gift(self, UGID: int) -> Gift | BadAPIRequest:
        """Возвращает информацию о подарке.

        :param UGID: Идентификатор подарка
        :type UGID: int
        """
        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"gift/{UGID}")

        if result["status_code"] == 200:
            return Gift.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def user_gifts(self, UUID: int) -> Gifts | BadAPIRequest:
        """Возвращает информацию о подарках игрока.

        :param UUID: Идентификатор игрока
        :type UUID: int
        """
        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"gifts/{UUID}")

        if result["status_code"] == 200:
            return Gifts.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def hero(self, UHID: int) -> Hero | BadAPIRequest:
        """Возвращает информацию о персонаже.

        :param UHID: Идентификатор персонажа
        :type UHID: int
        """
        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"hero/{UHID}")

        if result["status_code"] == 200:
            return Hero.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def star_box(self, UBID: int) -> StarBox | BadAPIRequest:
        """Возвращает информацию о StarBox.

        :param UBID: Идентификатор StarBox
        :type UBID: int
        """
        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"sb/{UBID}")

        if result["status_code"] == 200:
            return StarBox.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)



from json import dumps
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..main import CupLegend

from ..types import BadAPIRequest, BlockchainConfig as Config, BlockchainStats as Stats, Contract, NFT, Wallet, Rate
from ..types import Pools, Pool, TVL, TX, TXCreate, TXSign


class BlockchainMethods:
    def __init__(self, parent: "CupLegend"):
        self._parent = parent

    def config(self) -> Config | BadAPIRequest:
        """Возвращает конфигурацию блокчейна."""
        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"blockchain/config")

        if result["status_code"] == 200:
            return Config.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def stats(self) -> Stats | BadAPIRequest:
        """Возвращает статистику блокчейна."""
        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"blockchain/stats")

        if result["status_code"] == 200:
            return Stats.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def contract(self, ca: str) -> Contract | BadAPIRequest:
        """Возвращает указанный контракт.

        :param ca: Адрес контракта
        :type ca: str
        """
        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"blockchain/contract/{ca}")

        if result["status_code"] == 200:
            return Contract.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def nft(self, ca: str) -> NFT | BadAPIRequest:
        """Возвращает указанный NFT.

        :param ca: Адрес контракта
        :type ca: str
        """
        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"blockchain/nft/{ca}")

        if result["status_code"] == 200:
            return NFT.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def wallet(self, wallet: str) -> Wallet | BadAPIRequest:
        """Возвращает указанный кошелёк.

        :param wallet: Адрес кошелька
        :type wallet: str
        """
        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"blockchain/wallet/{wallet}")

        if result["status_code"] == 200:
            return Wallet.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def rate(self, pair_1: str, pair_2: str) -> Rate | BadAPIRequest:
        """Возвращает цены указанной пары.

        :param pair_1: Адрес первой пары
        :type pair_1: str
        :param pair_2: Адрес второй пары
        :type pair_2: str
        """
        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"blockchain/rate/{pair_1}/{pair_2}")

        if result["status_code"] == 200:
            return Rate.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def pools(self) -> Pools | BadAPIRequest:
        """Возвращает все доступные пулы."""
        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"blockchain/pools")

        if result["status_code"] == 200:
            return Pools.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def pool_by_id(self, pool_id: int) -> Pool | BadAPIRequest:
        """Возвращает указанный пул.

        :param pool_id: Идентификатор контракта
        :type pool_id: str
        """
        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"blockchain/pool/{pool_id}")

        if result["status_code"] == 200:
            return Pool.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def pool_by_pairs(self, pair_1: str, pair_2: str) -> Pool | BadAPIRequest:
        """Возвращает указанный пул.

        :param pair_1: Адрес первой пары
        :type pair_1: str
        :param pair_2: Адрес второй пары
        :type pair_2: str
        """
        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"blockchain/pool/{pair_1}/{pair_2}")

        if result["status_code"] == 200:
            return Pool.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def tvl(self, pair_1: str, pair_2: str, gems: bool = True) -> TVL | BadAPIRequest:
        """Возвращает TVL указанного пула.

        :param pair_1: Адрес первой пары
        :type pair_1: str
        :param pair_2: Адрес второй пары
        :type pair_2: str
        :param gems: Валюта TVL
        :type gems: bool
        """
        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"blockchain/tvl/{pair_1}/{pair_2}?gems={gems}")

        if result["status_code"] == 200:
            return TVL.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def get_tx(self, tx_id: int) -> TX | BadAPIRequest:
        """Возвращает указанную транзакцию.

        :param tx_id: Идентификатор транзакции
        :type tx_id: int
        """
        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"blockchain/tx/get/{tx_id}")

        if result["status_code"] == 200:
            return TX.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def create_tx(self, data: dict) -> TXCreate | BadAPIRequest:
        """Создаёт транзакцию с указанными мета-данными.

        :param data: Тело транзакции
        :type data: dict
        """
        # noinspection PyProtectedMember
        if not self._parent._token:
            raise Exception("Для использования этого метода требуется создать объект с действующим API-ключом.")

        # noinspection PyProtectedMember
        result = self._parent._api_call(method="post",
                                        url=f"blockchain/tx/create/{self._parent._token}",
                                        data=dumps(data))

        if result["status_code"] == 200:
            return TXCreate.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def sign_tx(self, tx_id: int, seed: str) -> TXSign | BadAPIRequest:
        """Подписывает указанную транзакцию.

        :param tx_id: Идентификатор транзакции
        :type tx_id: int
        :param seed: Сид-фраза кошелька
        :type seed: str
        """
        # noinspection PyProtectedMember
        if not self._parent._token:
            raise Exception("Для использования этого метода требуется создать объект с действующим API-ключом.")

        # noinspection PyProtectedMember
        result = self._parent._api_call(method="post",
                                        url=f"blockchain/tx/sign/{self._parent._token}",
                                        data=dumps({"tx_id": tx_id,
                                                    "seed": seed}))

        if result["status_code"] == 200:
            return TXSign.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)



from pydantic import BaseModel


class BadAPIRequest(BaseModel):
    status_code: int
    msg: str


class DateTime(BaseModel):
    date: str
    time: str


class GiftMetaData(BaseModel):
    collection: int
    link: str
    name: str
    ticker: str
    rare: str
    supply: int | str
    price: int


class GiftModel(BaseModel):
    name: str
    number: int
    chance: int
    upgraded: DateTime
    gift_link: str



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..main import CupLegend

from ..types import BadAPIRequest, CheckDelete, CheckCreate


class ChecksMethods:
    def __init__(self, parent: "CupLegend"):
        self._parent = parent

    def create(self, activations: int, reward: int) -> CheckCreate | BadAPIRequest:
        """Создаёт чек с указанной наградой для указанного количества активаций.

        :param activations: Количество активаций
        :type activations: int
        :param reward: Награда за активацию
        :type reward: int
        """
        # noinspection PyProtectedMember
        if not self._parent._token:
            raise Exception("Для использования этого метода требуется создать объект с действующим API-ключом.")

        # noinspection PyProtectedMember
        result = self._parent._api_call(method="post",
                                        url=f"checks/create/{self._parent._token}/{activations}/{reward}")

        if result["status_code"] == 200:
            return CheckCreate.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def delete(self, code: str) -> CheckDelete | BadAPIRequest:
        """Удаляет указанный чек.

        :param code: Код чека
        :type code: str
        """
        # noinspection PyProtectedMember
        if not self._parent._token:
            raise Exception("Для использования этого метода требуется создать объект с действующим API-ключом.")

        # noinspection PyProtectedMember
        result = self._parent._api_call(method="delete",
                                        url=f"checks/delete/{self._parent._token}/{code}")

        if result["status_code"] == 200:
            return CheckDelete.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)



from .base_types import *


class Info(BaseModel):
    status_code: int
    name: str
    id: int
    username: str
    news: str
    chat: str


class Stats(BaseModel):
    status_code: int
    players: int
    online: int
    chats: int
    duels: int
    cup_up: int
    trophies: int
    rubles: int
    tokens: int
    gems: int


class Account(BaseModel):
    class _Social(BaseModel):
        karma: int
        mark: str

    status_code: int
    UUID: int
    clan: int
    level: int
    trophies: int
    nickname: str
    privilege: str
    social: _Social
    reg_date: DateTime
    online: DateTime


class Clan(BaseModel):
    class _ClanMember(BaseModel):
        nickname: str
        date: str
        time: str
        stars: int

    status_code: int
    UCID: int
    name: str
    leader: int
    level: int
    create_date: DateTime
    description: str
    rating: int
    members: dict[str, _ClanMember]


class Gift(BaseModel):
    status_code: int
    owner: int
    sender: int
    received: DateTime
    meta: GiftMetaData
    model: GiftModel | dict


class Gifts(BaseModel):
    class _GiftInfo(BaseModel):
        sender: int
        received: DateTime
        meta: GiftMetaData
        model: GiftModel | dict

    status_code: int
    gifts: dict[str, _GiftInfo]


class Hero(BaseModel):
    status_code: int
    creator: int
    UHID: int
    name: str
    description: str
    religion: str
    wins: int
    loses: int
    level: int
    trophies: int
    create_date: DateTime


class StarBox(BaseModel):
    class _StarBoxMeta(BaseModel):
        experience: int
        gems: int
        trophies: int
        tokens: int
        fragments: int
        energy: int
        privilege: str
        rune: str
        artifact: str
        weapons: list[str]

    status_code: int
    opener: int
    rare: str
    date: DateTime
    link: str
    meta: _StarBoxMeta


class AppInfo(BaseModel):
    status_code: int
    UUID: int
    UAID: int
    name: str
    balance: int
    checks: int
    invoices: int


class AppInvoices(BaseModel):
    class _InvoiceInfo(BaseModel):
        payments: int
        amount: int

    status_code: int
    invoices: dict[str, _InvoiceInfo]


class GetStorage(BaseModel):
    status_code: int
    storage: dict


class SetStorage(BaseModel):
    status_code: int


class ClearStorage(BaseModel):
    status_code: int


class SendGift(BaseModel):
    status_code: int


class BlockchainConfig(BaseModel):
    status_code: int
    contract: str
    fee: float
    available_tx_types: list[str]
    price_channel: int
    max_token_supply: int


class BlockchainStats(BaseModel):
    status_code: int
    total_tx: int
    total_confirmed_tx: int
    tps: dict[str, float | None]
    tx_stats: dict[str, dict[str, int]]


class Contract(BaseModel):
    status_code: int
    master: str | None = None
    type: str
    init: float | int
    name: str
    ticker: str | None = None
    channel: int
    supply: float | int
    max_supply: float | None = None
    fee: float | None = None
    attributes: list[str] | None = None
    honeypot: bool | None = None
    honeypot_reverse: bool | None = None
    soulbound: bool | None = None
    verification: bool


class NFT(BaseModel):
    status_code: int
    contract: str
    collection: str
    sequence: int
    attributes: dict[str, str | int | float]
    format: str
    owner: str


class Wallet(BaseModel):
    status_code: int
    tokens: dict[str, float | int] | dict
    lp: dict[str, float | int] | dict
    nft: list[str] | list


class Rate(BaseModel):
    status_code: int
    first_rate: float | int
    second_rate: float | int


class Pools(BaseModel):
    status_code: int
    pools: dict[str, dict[str, list[list[dict[str, str | None]]] | dict[str, float | int]]]


class Pool(BaseModel):
    status_code: int
    pool_id: int
    pool_address: str
    pair_1: dict[str, str | float | int]
    pair_2: dict[str, str | float | int]


class TVL(BaseModel):
    status_code: int
    tvl: float | int


class TX(BaseModel):
    status_code: int
    status: int
    type: str
    wallet: str
    fee: float | int
    init: float
    launched: float | None = None
    finished: float | None = None
    duration: float | None = None
    token: str | None = None
    master: str | None = None
    new_owner: str | None = None
    gift_id: int | None = None
    market: str | float | int | None = None
    pool: str | None = None
    ca: str | None = None
    contract: str | None = None
    collection: str | None = None
    domain: str | None = None
    pair: str | None = None
    sent: float | int | None = None
    received: float | int | None = None
    receivers: list[str] | None = None
    message: str | None = None
    old_rate: float | int | None = None
    new_rate: float | int | None = None
    token_old_rate: float | int | None = None
    token_new_rate: float | int | None = None
    pair_old_rate: float | int | None = None
    pair_new_rate: float | int | None = None
    percent: float | int | None = None
    lot: str | int | None = None
    nft: str | None = None
    summa: int | float | None = None
    price: int | float | None = None
    summa_1: int | float | None = None
    summa_2: int | float | None = None
    pair_1: int | float | None = None
    pair_2: int | float | None = None
    lp: int | float | None = None


class TXCreate(BaseModel):
    status_code: int
    tx_id: int


class TXSign(BaseModel):
    status_code: int


class CheckCreate(BaseModel):
    status_code: int
    code: str
    link: str


class CheckDelete(BaseModel):
    status_code: int


class InvoiceInfo(BaseModel):
    class _Payment(BaseModel):
        date: DateTime
        summa: int
        comment: str

    status_code: int
    payed: bool
    payments: dict[str, dict[str, _Payment]] | dict


class InvoiceCreate(BaseModel):
    status_code: int
    code: str
    link: str


class InvoiceDelete(BaseModel):
    status_code: int



from .base import BaseMethods
from .info import InfoMethods
from .app import AppMethods
from .blockchain import BlockchainMethods
from .checks import ChecksMethods
from .invoices import InvoicesMethods



from .methods import *

from requests import request

class CupLegend:
    def __init__(self, api_key: str = None):
        """Создаёт объект для обращения к API-методам Cup Legend.

        :param api_key: API-ключ (опционально)
        :type api_key: str
        """
        self._url = "https://api.cuplegend.ru"
        self._token: str | None = api_key

        self.base: BaseMethods = BaseMethods(self)
        self.info: InfoMethods = InfoMethods(self)
        self.app: AppMethods = AppMethods(self)
        self.blockchain: BlockchainMethods = BlockchainMethods(self)
        self.checks: ChecksMethods = ChecksMethods(self)
        self.invoices: InvoicesMethods = InvoicesMethods(self)

    def _api_call(self, url: str = "", method: str = "get", data: str = None) -> dict:
        return request(method=method, url=f"{self._url}/{url}", data=data).json()



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..main import CupLegend

from ..types import Info, BadAPIRequest, Stats


class BaseMethods:
    def __init__(self, parent: "CupLegend"):
        self._parent = parent

    def info(self) -> Info | BadAPIRequest:
        """Возвращает информацию о боте."""
        # noinspection PyProtectedMember
        result = self._parent._api_call()

        if result["status_code"] == 200:
            return Info.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def stats(self) -> Stats | BadAPIRequest:
        """Возвращает статистику бота."""
        # noinspection PyProtectedMember
        result = self._parent._api_call(url="stats")

        if result["status_code"] == 200:
            return Stats.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..main import CupLegend

from ..types import BadAPIRequest, InvoiceCreate, InvoiceDelete, InvoiceInfo


class InvoicesMethods:
    def __init__(self, parent: "CupLegend"):
        self._parent = parent

    def get(self, code: str) -> InvoiceInfo | BadAPIRequest:
        """Возвращает указанный счёт.

        :param code: Код счёта
        :type code: str
        """
        # noinspection PyProtectedMember
        if not self._parent._token:
            raise Exception("Для использования этого метода требуется создать объект с действующим API-ключом.")

        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"invoices/get/{self._parent._token}/{code}")

        if result["status_code"] == 200:
            return InvoiceInfo.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def create(self, payments: int, summa: int, comment: bool = False) -> InvoiceCreate | BadAPIRequest:
        """Создаёт счёт с указанной суммой оплаты для указанного количества оплат.

        :param payments: Количество оплат
        :type payments: int
        :param summa: Сумма оплаты
        :type summa: int
        :param comment: Комментарии к оплате
        :type comment: bool
        """
        # noinspection PyProtectedMember
        if not self._parent._token:
            raise Exception("Для использования этого метода требуется создать объект с действующим API-ключом.")

        # noinspection PyProtectedMember
        result = self._parent._api_call(method="post",
                                        url=f"invoices/create/{self._parent._token}/{payments}/{summa}?comment={comment}")

        if result["status_code"] == 200:
            return InvoiceCreate.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def delete(self, code: str) -> InvoiceDelete | BadAPIRequest:
        """Удаляет указанный счёт.

        :param code: Код счёта
        :type code: str
        """
        # noinspection PyProtectedMember
        if not self._parent._token:
            raise Exception("Для использования этого метода требуется создать объект с действующим API-ключом.")

        # noinspection PyProtectedMember
        result = self._parent._api_call(method="delete",
                                        url=f"invoices/delete/{self._parent._token}/{code}")

        if result["status_code"] == 200:
            return InvoiceDelete.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..main import CupLegend


from json import dumps

from ..types import BadAPIRequest, AppInvoices, SendGift, AppInfo, GetStorage, SetStorage, ClearStorage


class StorageMethods:
    def __init__(self, parent: "AppMethods"):
        self._parent = parent

    def get(self) -> GetStorage | BadAPIRequest:
        """Возвращает хранилище приложения."""
        # noinspection PyProtectedMember
        if not self._parent._parent._token:
            raise Exception("Для использования этого метода требуется создать объект с действующим API-ключом.")

        # noinspection PyProtectedMember
        result = self._parent._parent._api_call(url=f"app/storage/get/{self._parent._parent._token}")

        if result["status_code"] == 200:
            return GetStorage.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def set(self, storage: dict) -> SetStorage | BadAPIRequest:
        """Обновляет хранилище приложения.

        :param storage: Словарь (dict) с данными
        :type storage: dict
        """
        # noinspection PyProtectedMember
        if not self._parent._parent._token:
            raise Exception("Для использования этого метода требуется создать объект с действующим API-ключом.")

        # noinspection PyProtectedMember
        result = self._parent._parent._api_call(method="post",
                                                url=f"app/storage/set/{self._parent._parent._token}",
                                                data=dumps(storage))

        if result["status_code"] == 200:
            return SetStorage.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def clear(self) -> ClearStorage | BadAPIRequest:
        """Очищает хранилище приложения."""
        # noinspection PyProtectedMember
        if not self._parent._parent._token:
            raise Exception("Для использования этого метода требуется создать объект с действующим API-ключом.")

        # noinspection PyProtectedMember
        result = self._parent._parent._api_call(method="delete",
                                                url=f"app/storage/clear/{self._parent._parent._token}")

        if result["status_code"] == 200:
            return ClearStorage.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)


class AppMethods:
    def __init__(self, parent: "CupLegend"):
        self._parent = parent

        self.storage = StorageMethods(self)

    def myself(self) -> AppInfo | BadAPIRequest:
        """Возвращает информацию о приложении."""
        # noinspection PyProtectedMember
        if not self._parent._token:
            raise Exception("Для использования этого метода требуется создать объект с действующим API-ключом.")

        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"app/myself/{self._parent._token}")

        if result["status_code"] == 200:
            return AppInfo.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def invoices(self) -> AppInvoices | BadAPIRequest:
        """Возвращает информацию о созданных счетах."""
        # noinspection PyProtectedMember
        if not self._parent._token:
            raise Exception("Для использования этого метода требуется создать объект с действующим API-ключом.")

        # noinspection PyProtectedMember
        result = self._parent._api_call(url=f"app/invoices/{self._parent._token}")

        if result["status_code"] == 200:
            return AppInvoices.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)

    def send_gift(self, UGID: int, UUID: int) -> SendGift | BadAPIRequest:
        """Отправляет указанный подарок на указанный аккаунт."""
        # noinspection PyProtectedMember
        if not self._parent._token:
            raise Exception("Для использования этого метода требуется создать объект с действующим API-ключом.")

        # noinspection PyProtectedMember
        result = self._parent._api_call(method="post",
                                        url=f"app/send_gift/{self._parent._token}/{UGID}/{UUID}")

        if result["status_code"] == 200:
            return SendGift.model_validate(result)
        else:
            return BadAPIRequest.model_validate(result)
